# import geom.distance as distance
# import geom.elevation as elevation
# import geom.geo_pt as geo_pt
# import geom.line as line
# import geom.pointable as pointable
# import geom.pt as pt
# import geom.shape as shape

# __all__ = ["distance", "elevation", "geo_pt", "line", "pointable", "pt", "shape"]


from __future__ import annotations
from os.path import join, dirname
from typing import Any, Dict, List, Optional

from shapely import geometry
import fiona
import geopandas as gpd

from color.color import Color
from geom.geo_pt import GeoPt
from geom.line import Line
import locations.mrt.platform

class MRTLine(Line):
    name: str
    color: Color
    platforms: List[locations.mrt.platform.Platform]
    
    def __init__(self, name: str, color: Color, line: Optional[List[GeoPt]]):
        super(Line, self).__init__(line)
        self.name = name
        self.color = color
        self.platforms = []
        
    def __str__(self) -> str:
        return f"<MRTLine: {self.name}>"
    
    def __repr__(self) -> str:
        return f"<MRTLine: {self.name}>"
    
def get_mrt_lines() -> Dict[str, MRTLine]:
    MRT_LINES = {}
    
    gpd.io.file.fiona.drvsupport.supported_drivers['KML'] = 'rw'
    lines = gpd.read_file(join(dirname(__file__), "assets/mrt-line.kml"))
    lines.loc[5:8,"geometry"] = lines.loc[5:8, "geometry"].boundary
    lines.drop(columns=["Description"], inplace=True)
    lines.Name = lines.Name.str.extract("([^L]+)")
    lines_dict = lines.set_index("Name").to_dict("index")
    lines_dict["CE"] = {}
    lines_dict["CE"]["geometry"] = geometry.LineString(list(GeoPt(x[1], x[0]) for x in lines_dict["CC"]["geometry"].coords[346:][::-1]))
    
    color_mapping = {
        "EW": "009645",
        "CG": "009645",
        "NS": "D42E12",
        "NE": "9900AA",
        "CC": "FA9E0D",
        "CE": "FA9E0D",
        "DT": "005EC4",
        "TE": "9D5B25",
        "JS": "0099AA",
        "JE": "0099AA",
        "JW": "0099AA",
        "CR": "97C616",
        "CP": "97C616",
        "SE": "748477",
        "SW": "748477",
        "PE": "748477",
        "PW": "748477",
        "BP": "748477",
    }
    
    for name, data in lines_dict.items():
        MRT_LINES[name] = MRTLine(name,
                                  Color.from_hex(color_mapping[name]),
                                  [GeoPt(x[1], x[0]) for x in data["geometry"].coords])
    return MRT_LINES
    


from __future__ import annotations
from os.path import dirname, join
from typing import Any, Callable, Dict, List, Optional, Tuple

import pandas as pd
import pickle

from geom.elevation import ElevationMap
from geom.shape import Shape
from .location import Location, Locations

class BusStop(Location):
  """
  Object to encapsulate a Bus Stop in Singapore.

  Fields:
    services (List[str]): The bus services that go to this Bus Stop.
    road_name (str): The name of the road that this bus stop is on.
  """
  services: List[str]
  road_name: str

  def __init__(self, name: str, **kwargs):
    fields = ["lat", "lon", "shape", "services", "road_name"]
    self._try_setter(fields, kwargs)
    super().__init__(name, lat=self.lat, lon=self.lon, shape=self.shape)

class BusStops(Locations):
  _FIELD_MAP = {
    
  }

  def __init__(self, *bus_stops: BusStop, name: str="busstop"):
    super().__init__(*bus_stops, name=name)

  @staticmethod
  def get(blanks: bool=False, offline: bool=True) -> BusStops:
    raw_df = BusStops._get_data_handler(offline)
    data_dict = BusStops._get_data_cleaning(blanks)(raw_df)
    bus_stops = BusStops._get_data_compiling(data_dict)
    return BusStops(*bus_stops)

  @staticmethod
  def _get_data_handler(offline: bool) -> pd.DataFrame:
    if offline:
      return pd.read_csv(join(dirname(__file__), "assets/bus.csv"))
    print("Retrieving 'Bus' from Sheets...")
    raw_df = Locations.get_sheet("Bus")
    print("Retrieved.")
    return raw_df

  @staticmethod
  def _get_data_cleaning(blanks: bool=True) -> Callable[[pd.DataFrame], Dict[str, Any]]:
    def convert_into_dict(df: pd.DataFrame) -> Dict[str, Any]:
      buses_dict = {}
      for bus_stop, frame in df.groupby("BusStopCode"):
        buses_dict[bus_stop] = {
          "lat": frame.Latitude.to_list()[0],
          "lon": frame.Longitude.to_list()[0],
          "road_name": frame.RoadName.to_list()[0],
          "services": list(set(frame.ServiceNo.to_list()))
        }
      return buses_dict
    return convert_into_dict
      
  @staticmethod
  def _get_data_compiling(data_dict: Dict[str, Any]) -> List[BusStop]:
    bus_stops: List[BusStop] = []
    for name, info in data_dict.items():
      if ElevationMap.in_singapore(info["lat"], info["lon"]):
        bus_stops.append(BusStop(name, **BusStops._field_map(info)))
    return bus_stops

  @staticmethod
  def _field_map(d: Dict[str, Any]) -> Dict[str, Any]:
    for old_field, new_field in BusStops._FIELD_MAP.items():
      d[new_field] = d.pop(old_field)
    return d



import geopandas as gpd
import fiona
from geom.line import Line

class Road(Line):
    gpd.io.file.fiona.drvsupport.supported_drivers["KML"] = "rw"
    roads = gpd.read_file()

class RoadNetwork:
    _PATH_TO_ROAD_NETWORK = join(dirname(__file__), "assets/singapore-elevation.png")
    
    @classmethod
    def _get_roads(cls) -> Dict[str, Road]
